Type: Package
Package: dvir
Title: Disaster Victim Identification
Version: 3.4.2
Authors@R: c(
    person("Magnus Dehli", "Vigeland", , "m.d.vigeland@medisin.uio.no", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9134-4962")),
    person("Thore", "Egeland", , "thore.egeland@nmbu.no", role = "aut",
           comment = c(ORCID = "0000-0002-3465-8885"))
  )
Description: Joint DNA-based disaster victim identification (DVI), as
    described in Vigeland and Egeland (2021)
    <doi:10.21203/rs.3.rs-296414/v1>. Identification is performed by
    optimising the joint likelihood of all victim samples and reference
    individuals. Individual identification probabilities, conditional on
    all available information, are derived from the joint solution in the
    form of posterior pairing probabilities. 'dvir' is part of the
    'pedsuite' collection of packages for pedigree analysis.
License: GPL-3
URL: https://github.com/magnusdv/dvir
BugReports: https://github.com/magnusdv/dvir/issues
Depends: pedtools (>= 2.11.0), R (>= 4.2.0)
Imports: forrel (>= 1.9.0), mirai, pedFamilias, pedprobr (>= 1.1.0),
        ribd, verbalisr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Encoding: UTF-8
Language: en-GB
LazyData: true
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-30 13:44:09 UTC; magnu
Author: Magnus Dehli Vigeland [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9134-4962>),
  Thore Egeland [aut] (ORCID: <https://orcid.org/0000-0002-3465-8885>)
Maintainer: Magnus Dehli Vigeland <m.d.vigeland@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-06-30 14:50:02 UTC
Built: R 4.6.0; ; 2026-06-30 17:43:51 UTC; unix
