# Generated by roxygen2: do not edit by hand

S3method(print,dviData)
export(Bmarginal)
export(addVictims)
export(amDrivenDVI)
export(checkDVI)
export(compactJointRes)
export(directMatch)
export(dviCompare)
export(dviData)
export(dviGridSize)
export(dviJoint)
export(dviSim)
export(dviSolve)
export(excludePairing)
export(exclusionMatrix)
export(expand.grid.nodup)
export(familias2dvir)
export(findExcluded)
export(findNonidentifiable)
export(findUndisputed)
export(formatSummary)
export(generatePairings)
export(getDatabase)
export(getFamily)
export(getSimpleFams)
export(jointDVI)
export(mergePM)
export(ncomb)
export(pairwiseLR)
export(plotDVI)
export(plotSolution)
export(plotUndisputed)
export(relabelDVI)
export(sequentialDVI)
export(setPairing)
export(subsetDVI)
export(swapOrientation)
import(pedtools)
importFrom(forrel,findExclusions)
importFrom(forrel,profileSim)
importFrom(grDevices,dev.new)
importFrom(graphics,grconvertX)
importFrom(graphics,grconvertY)
importFrom(graphics,layout)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,rect)
importFrom(pedprobr,likelihood)
importFrom(stats,aggregate)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(verbalisr,verbalise)
