#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import pedtools
#' @importFrom pbapply pblapply pboptions
#' @importFrom parallel makeCluster stopCluster detectCores parLapply clusterEvalQ clusterExport
## usethis namespace: end
NULL
